import argparse
import logging
import sys
import re

import boto3

from index_registry.utils.config import Config

_logger = logging.getLogger(__name__)


def restore(args, config):
    if args.registry_restore_source_leases_table and args.registry_restore_target_leases_table:
        restore_ddb(args, config)
        print(f"data loading for org {args.registry_restore_org_key} finished")


def restore_ddb(args, config):
    region = config.get("aws", "default_region")
    source_dynamodb = boto3.client("dynamodb", region_name=region)
    target_dynamodb = boto3.client("dynamodb", region_name=region)

    org_key = args.registry_restore_org_key
    source_leases_table = args.registry_restore_source_leases_table
    target_leases_table = args.registry_restore_target_leases_table
    print(f'Start restoring leases table for org: {org_key}')

    scan_params = {'TableName': source_leases_table}
    last_evaluated_key = None

    while True:
        if last_evaluated_key:
            scan_params['ExclusiveStartKey'] = last_evaluated_key
        resp = source_dynamodb.scan(**scan_params)
        for item in resp['Items']:
            s3_path = item['index_path']['S']
            org = _get_org_by_path(s3_path)
            if org and org == org_key:
                target_dynamodb.put_item(
                    TableName=target_leases_table,
                    Item=item
                )
        last_evaluated_key = resp.get('LastEvaluatedKey')
        if not last_evaluated_key:
            break


# raw: indexer://indexer/class=default/org=_09MUVJI/partition=0/type=raw/tstamp=0/size=0/index
# merging gs://dev-solr-index-bucket/class=default/org=7PYMT6QP/partition=0/type=merging/tstamp=0/size=0/placeholder
# optimized: gs://dev-solr-index-bucket/expire=2023-10-04/class=longTerm/org=RYET6V26E/partition=0/type=optimized/tstamp=18755aea0ce/size=0/index
def _get_org_by_path(s3_path: str) -> str:
    match = re.search(r'/org=([^/]+)', s3_path)
    if match:
        return match.group(1)
    return None


# def get_parser():
#     parser = argparse.ArgumentParser(description="Loader for index registry data recovery")
#     parser.add_argument('--registry-restore-source-leases-table', default='dev__index_leases_backup', help='source table for leases', type=str)
#     parser.add_argument('--registry-restore-target-leases-table', default='dev__index_leases', help='target table for leases', type=str)
#     parser.add_argument('--registry-restore-org-key', default="_iVvN8Kp", help='use valid org key', type=str)
#     parser.add_argument('--aws-region', default='us-east-1', type=str)
#     # parser.add_argument('--registry-restore-source-leases-table', help='source table for leases', type=str, required=False, default=False)
#     # parser.add_argument('--registry-restore-target-leases-table', help='target table for leases', type=str, required=False, default=False)
#     # parser.add_argument('--registry-restore-org-key', help='use valid org key', type=str, required=False, default=False)
#
#     parsed_args = parser.parse_args()
#     return parsed_args
#
#
# def _get_config(args):
#     config = Config.create()
#     config.set('aws', 'default_region', args.aws_region)
#     config.set('lease_dynamodb', 'dynamodb_table', args.registry_restore_target_leases_table)
#     return config
#
# def main():
#     args = get_parser()
#     args_dict = args.__dict__
#     d = {True for k, v in args_dict.items() if v is None}
#     is_missing = len(d)
#     if len(args_dict) == 0 or is_missing:
#         raise ValueError(f"Some args are missing \n args: {args}")
#     config = _get_config(args)
#     restore(args, config)
#
# # this is only for local testing
# if __name__ == "__main__":
#     sys.exit(main())

